use leptos::*;
use serde::{Deserialize, Serialize};
use serde_json::Value;

#[derive(Clone, Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase", untagged)]
pub enum RichTextNode {
    Block {
        node_type: String,
        // data: Option<serde_json::Value>,
        content: Vec<RichTextNode>,
    },
    Text {
        node_type: String,
        // data: Option<serde_json::Value>,
        value: String,
        // marks: Vec<Mark>,
    },
}

#[derive(Clone, Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TextBlock {
    node_type: String,
    // data: Option<serde_json::Value>,
    value: String,
    // marks: Vec<Mark>,
}


#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct Mark {
    #[serde(rename = "type")]
    pub mark_type: String
}

#[derive(Debug, Clone)]
struct NodeHandler {
    node_type: &'static str,
    function: fn (node: &Value) -> HtmlElement<html::AnyElement>,
}

const NODE_HANDLERS: [NodeHandler; 12] = [
    NodeHandler { node_type: "document", function: document_handler },
    NodeHandler { node_type: "paragraph", function: paragraph_handler },
    NodeHandler { node_type: "text", function: text_handler },
    NodeHandler { node_type: "heading-1", function: heading_1_handler },
    NodeHandler { node_type: "heading-2", function: heading_2_handler },
    NodeHandler { node_type: "heading-3", function: heading_3_handler },
    NodeHandler { node_type: "heading-4", function: heading_4_handler },
    NodeHandler { node_type: "heading-5", function: heading_5_handler },
    NodeHandler { node_type: "heading-6", function: heading_6_handler },
    NodeHandler { node_type: "hr", function: hr_handler },
    NodeHandler { node_type: "blockquote", function: quote_handler },
    NodeHandler { node_type: "embedded-asset-block", function: asset_handler },
];

pub fn document_handler(node: &Value) -> HtmlElement<html::AnyElement> {
    let children: Vec<Value> = serde_json::from_value(node["content"].clone()).unwrap();
    view! {
        <div class="rich-text-document">
            {render_children(&children)}
        </div>
    }.into()
}

fn paragraph_handler(node: &Value) -> HtmlElement<html::AnyElement> {
    let children: Vec<Value> = serde_json::from_value(node["content"].clone()).unwrap();
    view! {
        <p>
            {render_children(&children)}
        </p>
    }.into()
}

fn text_handler(node: &Value) -> HtmlElement<html::AnyElement> {
    let block: TextBlock = serde_json::from_value(node.clone()).unwrap();
    let text = block.value.replace("\n", "<br/>");
    view! { <span inner_html=text>{}</span> }.into()
}

fn heading_1_handler(node: &Value) -> HtmlElement<html::AnyElement> {
    let children: Vec<Value> = serde_json::from_value(node["content"].clone()).unwrap();
    view! { <h1> {render_children(&children)} </h1> }.into()
}

fn heading_2_handler(node: &Value) -> HtmlElement<html::AnyElement> {
    let children: Vec<Value> = serde_json::from_value(node["content"].clone()).unwrap();
    view! { <h2> {render_children(&children)} </h2> }.into()
}

fn heading_3_handler(node: &Value) -> HtmlElement<html::AnyElement> {
    let children: Vec<Value> = serde_json::from_value(node["content"].clone()).unwrap();
    view! { <h3> {render_children(&children)} </h3> }.into()
}

fn heading_4_handler(node: &Value) -> HtmlElement<html::AnyElement> {
    let children: Vec<Value> = serde_json::from_value(node["content"].clone()).unwrap();
    view! { <h4> {render_children(&children)} </h4> }.into()
}

fn heading_5_handler(node: &Value) -> HtmlElement<html::AnyElement> {
    let children: Vec<Value> = serde_json::from_value(node["content"].clone()).unwrap();
    view! { <h5> {render_children(&children)} </h5> }.into()
}

fn heading_6_handler(node: &Value) -> HtmlElement<html::AnyElement> {
    let children: Vec<Value> = serde_json::from_value(node["content"].clone()).unwrap();
    view! { <h6> {render_children(&children)} </h6> }.into()
}

fn hr_handler(_node: &Value) -> HtmlElement<html::AnyElement> {
    view! { <hr/> }.into()
}

fn quote_handler(node: &Value) -> HtmlElement<html::AnyElement> {
    let children: Vec<Value> = serde_json::from_value(node["content"].clone()).unwrap();
    view! { <blockquote> {render_children(&children)} </blockquote> }.into()
}


fn asset_handler(_node: &Value) -> HtmlElement<html::AnyElement> {
    // let data = node["data"];
    // let img_url = format!(
    //     "https://images.ctfassets.net/e3magj9g6dp1/14q5L7K0BCol1gx0aCSCck/d1f69bfa404efed6a2dcc71401bbc16d/P5310039-1-2.jpg?w=1600&q=50&fm=avif"
    // );
    view! { <img>  </img> }.into()
}

fn empty_handler() -> HtmlElement<html::AnyElement> {
    view! { <div/> }.into()
}

fn node_dispatcher(node: &Value) -> HtmlElement<html::AnyElement> {
    for handler in NODE_HANDLERS.iter() {
        let node_type = node["nodeType"].clone();
        if handler.node_type == node_type {
            return (handler.function)(node)
        }
    }

    empty_handler()
}

fn render_children(children: &Vec<Value>) -> Vec<HtmlElement<html::AnyElement>> {
    children.iter().map(|child| node_dispatcher(child)).collect()
}

// pub fn render_rich_text(rich_text: Value) -> HtmlElement<html::AnyElement> {
//     node_dispatcher(&rich_text)
// }

// fn _document_handler(node: &RichTextNode) -> HtmlElement<html::AnyElement> {
//     let empty_vec: Vec<RichTextNode> = Vec::new();
//     let children = match node {
//         RichTextNode::Block{node_type: _, content} => content,
//         // RichTextNode::Block{node_type: _, data: _, content} => content,
//         _ => &empty_vec,
//     };
//     view! {
//         <div class="rich-text-document">
//             {_render_children(children)}
//         </div>
//     }.into()
// }

// fn _text_handler(node: &RichTextNode) -> HtmlElement<html::AnyElement> {
//     match node {
//         RichTextNode::Text{node_type: _, value} => view! { <span>{value}</span> }.into(),
//         // RichTextNode::Text{node_type: _, data: _, value, marks} => view! { <span>{value}</span> }.into(),
//         _ => view! { <span>Error</span> }.into(),
//     }
// }

// fn _node_dispatcher(node: &RichTextNode) -> HtmlElement<html::AnyElement> {
//     for handler in NODE_HANDLERS.iter() {
//         let node_type = match node.clone() {
//             // RichTextNode::Block{node_type, data: _, content: _} => node_type,
//             // RichTextNode::Text{node_type, data: _, value: _, marks: _} => node_type,
//             RichTextNode::Block{node_type, content: _} => node_type,
//             RichTextNode::Text{node_type, value: _} => node_type,
//         };
//         if handler.node_type == node_type {
//             // return (handler.function)(node)
//         }
//     }

//     empty_handler()
// }

// fn _render_children(children: &Vec<RichTextNode>) -> Vec<HtmlElement<html::AnyElement>> {
//     children.iter().map(|child| _node_dispatcher(child)).collect()
// }
